static const char *RcsId = "$Header:  $";
//+=============================================================================
//
// file :         CS410LightController.cpp
//
// description :  C++ source for the CS410LightController and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                CS410LightController are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author:  $
//
// $Revision:  $
//
// $Log:  $
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name      |  Method name
//	----------------------------------------
//  State             |  dev_state()
//  Status            |  dev_status()
//  SetChannel0Power  |  set_channel0_power()
//  SetChannel1Power  |  set_channel1_power()
//
//===================================================================


#include <tango.h>
#include <CS410LightController.h>
#include <CS410LightControllerClass.h>

namespace CS410LightController_ns
{

//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::CS410LightController(string &s)
// 
// description : 	constructor for simulated CS410LightController
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
CS410LightController::CS410LightController(Tango::DeviceClass *cl,string &s)
:Tango::Device_4Impl(cl,s.c_str())
{
	init_device();
}

CS410LightController::CS410LightController(Tango::DeviceClass *cl,const char *s)
:Tango::Device_4Impl(cl,s)
{
	init_device();
}

CS410LightController::CS410LightController(Tango::DeviceClass *cl,const char *s,const char *d)
:Tango::Device_4Impl(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void CS410LightController::delete_device()
{
	//	Delete device's allocated object
	delete serial_device;
	serial_device = NULL;
}

//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void CS410LightController::init_device()
{
	INFO_STREAM << "CS410LightController::CS410LightController() create device " << device_name << endl;

	serial_device = NULL;
	
	attr_Channel0Power_read = &channel0Power;
	attr_Channel1Power_read = &channel1Power;
	
	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	
	if (serialDeviceConnectionString.empty())
	{
		TangoSys_OMemStream os;
		os << "Error, serialDeviceConnectionString is empty";
		DEBUG_STREAM << os.str().c_str() << endl;
		
		set_state(Tango::FAULT);
		set_status(os.str());
		
		Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
		FATAL_STREAM << "Fatal Error: serialDeviceConnectionString is empty!!!" << endl;
	}
	
	if (defaultChannel0Power < CHANNEL_POWER_MIN || defaultChannel0Power > CHANNEL_POWER_MAX)
	{
		DEBUG_STREAM << "Error, defaultChannel0Power is invalid, will set it to 0" << endl;
		
		defaultChannel0Power = 0;
	}
	
	if (defaultChannel1Power < CHANNEL_POWER_MIN || defaultChannel1Power > CHANNEL_POWER_MAX)
	{
		DEBUG_STREAM << "Error, defaultChannel1Power is invalid, will set it to 0" << endl;
		
		defaultChannel1Power = 0;
	}
	
	// create the TangoDeviceConnection object
	serial_device = new TangoDeviceConnection(serialDeviceConnectionString);
	
	try
	{
		// start the connection to the device: this one sets the comm parameters does synchronization
		Tango::DeviceProxy * dev = get_serial_device();
		
		// set default power for all devices
		set_power_for_channel(0, defaultChannel0Power);
		set_power_for_channel(1, defaultChannel1Power);
		
		// update power for channels
		//update_power_for_channel(0);
		//update_power_for_channel(1);
		
		set_state(Tango::ON);
		set_status("Inited");
	}
	catch (...)
	{
		DEBUG_STREAM << "Could not connect to the CS410LightController, or error while synchronizing" << endl;
		
		serial_device->disconnect();
		
		channel0Power = -1;
		channel1Power = -1;
		
		set_state(Tango::FAULT);
		set_status("Could not connect to the CS410LightController, or error while synchronizing");
		FATAL_STREAM << "Could not connect to the CS410LightController, or error while synchronizing" << endl;
	}
}


//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void CS410LightController::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("SerialDeviceConnectionString"));
	dev_prop.push_back(Tango::DbDatum("DefaultChannel0Power"));
	dev_prop.push_back(Tango::DbDatum("DefaultChannel1Power"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	CS410LightControllerClass	*ds_class =
		(static_cast<CS410LightControllerClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize SerialDeviceConnectionString from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serialDeviceConnectionString;
	else {
		//	Try to initialize SerialDeviceConnectionString from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serialDeviceConnectionString;
	}
	//	And try to extract SerialDeviceConnectionString value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialDeviceConnectionString;

	//	Try to initialize DefaultChannel0Power from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultChannel0Power;
	else {
		//	Try to initialize DefaultChannel0Power from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultChannel0Power;
	}
	//	And try to extract DefaultChannel0Power value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultChannel0Power;

	//	Try to initialize DefaultChannel1Power from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultChannel1Power;
	else {
		//	Try to initialize DefaultChannel1Power from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultChannel1Power;
	}
	//	And try to extract DefaultChannel1Power value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultChannel1Power;



	//	End of Automatic code generation
	//------------------------------------------------------------------
	Tango::DbData data_put;
	int	ii = -1;
	if (dev_prop[++ii].is_empty()==true)
	{
		Tango::DbDatum	property("SerialDeviceConnectionString");
		property	<<	serialDeviceConnectionString;
		data_put.push_back(property);
	}
	if (dev_prop[++ii].is_empty()==true)
	{
		Tango::DbDatum	property("DefaultChannel0Power");
		property	<<	defaultChannel0Power;
		data_put.push_back(property);
	}
	if (dev_prop[++ii].is_empty()==true)
	{
		Tango::DbDatum	property("DefaultChannel1Power");
		property	<<	defaultChannel1Power;
		data_put.push_back(property);
	}
	
	get_db_device()->put_property(data_put);

}
//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void CS410LightController::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void CS410LightController::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "CS410LightController::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::read_Channel0Power
// 
// description : 	Extract real attribute values for Channel0Power acquisition result.
//
//-----------------------------------------------------------------------------
void CS410LightController::read_Channel0Power(Tango::Attribute &attr)
{
	DEBUG_STREAM << "CS410LightController::read_Channel0Power(Tango::Attribute &attr) entering... "<< endl;
	
	attr.set_value(attr_Channel0Power_read);
}

//+----------------------------------------------------------------------------
//
// method : 		CS410LightController::read_Channel1Power
// 
// description : 	Extract real attribute values for Channel1Power acquisition result.
//
//-----------------------------------------------------------------------------
void CS410LightController::read_Channel1Power(Tango::Attribute &attr)
{
	DEBUG_STREAM << "CS410LightController::read_Channel1Power(Tango::Attribute &attr) entering... "<< endl;
	
	attr.set_value(attr_Channel1Power_read);
}



//+------------------------------------------------------------------
/**
 *	method:	CS410LightController::set_channel0_power
 *
 *	description:	method to execute "SetChannel0Power"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void CS410LightController::set_channel0_power(Tango::DevLong argin)
{
	DEBUG_STREAM << "CS410LightController::set_channel0_power(): entering... !" << endl;

	//	Add your own code to control device here
	
	if (argin < CHANNEL_POWER_MIN || argin > CHANNEL_POWER_MAX)
	{
		TangoSys_OMemStream os;
		os << "Error: power must be between 0 and 1000!";
		DEBUG_STREAM << os.str().c_str() << endl;
		ERROR_STREAM << __func__<< ": "<< os.str().c_str() << endl;
		
		set_state(Tango::FAULT);
		set_status(os.str());
		
		Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
	}
	
	set_power_for_channel(0, argin);
}

//+------------------------------------------------------------------
/**
 *	method:	CS410LightController::set_channel1_power
 *
 *	description:	method to execute "SetChannel1Power"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void CS410LightController::set_channel1_power(Tango::DevLong argin)
{
	DEBUG_STREAM << "CS410LightController::set_channel1_power(): entering... !" << endl;

	//	Add your own code to control device here
	
	if (argin < CHANNEL_POWER_MIN || argin > CHANNEL_POWER_MAX)
	{
		TangoSys_OMemStream os;
		os << "Error: power must be between 0 and 1000!";
		DEBUG_STREAM << os.str().c_str() << endl;
		ERROR_STREAM << __func__<< ": "<< os.str().c_str() << endl;
		
		set_state(Tango::FAULT);
		set_status(os.str());
		
		Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
	}
	
	set_power_for_channel(1, argin);

}

void CS410LightController::set_power_for_channel(int channel, Tango::DevLong power) throw (Tango::DevFailed)
{
	char buff[256];
	snprintf(buff, 256, COMMAND_SET_CHANNEL_POWER, channel, power);
	write_string(get_serial_device(), buff);
	
	// get and parse response (this one may throw an exception)
	
	Tango::DevLong new_power = get_and_parse_light_controller_power_response();
	
	if (channel == 0)
	{
		channel0Power = new_power;
	}
	else
	{
		channel1Power = new_power;
	}
	
	set_state(Tango::ON);
	set_status("Power set");
	INFO_STREAM << "Power set for channel" << channel<<" with value "<<power<<endl;
}

void CS410LightController::update_power_for_channel(int channel) throw (Tango::DevFailed)
{
	// send "read channel power" command
	
	char buff[256];
	snprintf(buff, 256, COMMAND_GET_CHANNEL_POWER, channel);
	write_string(get_serial_device(), buff);
	
	// get and parse response (this one may throw an exception
	
	Tango::DevLong power = get_and_parse_light_controller_power_response();
	
	if (channel == 0)
	{
		channel0Power = power;
	}
	else
	{
		channel1Power = power;
	}
	
	set_state(Tango::ON);
	set_status("Power read");
	INFO_STREAM << "Power read from channel" << channel<<" with value "<<power<<endl;
	
}

Tango::DevLong CS410LightController::get_and_parse_light_controller_power_response() throw (Tango::DevFailed)
{
	string response = read_line(get_serial_device()); //try_read_n_chars(get_serial_device(), CHANNEL_POWER_REPLY_LEN);
	
	// flush remaining input buffer (if any)
	
	flush_read_buffer(get_serial_device());
	
	// parse response
	
	if (response.size() < CHANNEL_POWER_REPLY_LEN || response.at(0) != 'U')
	{
		TangoSys_OMemStream os;
		os << "Error, LightController did not answer with a known response!";
		DEBUG_STREAM << os.str().c_str() << endl;
				
		set_state(Tango::FAULT);
		set_status(os.str());
				
		Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
		ERROR_STREAM << "Error: LightController did not answer with a known response!" << endl;
	}
	
	// everything looks ok, update tango attributes
	
	string power_str = response.substr(2,4);
	Tango::DevLong power = strtol(power_str.c_str(), NULL, 10);

	return power;
}

string CS410LightController::read_line(Tango::DeviceProxy *dev) throw (Tango::DevFailed)
{
	try
	{
		DEBUG_STREAM << "Read line (start)" << endl;
		
		string response;
		Tango::DeviceData data_read = dev->command_inout("DevSerReadLine");
		data_read >> response;
		
		DEBUG_STREAM << "Read line (" << response.size() << "): " << response.c_str() << endl;
		DEBUG_STREAM << "Read line (end)" << endl;
	
		return response;
	}
	catch(...)
	{
		DEBUG_STREAM << "Could not read from the CS410LightController!" << endl;
		
		serial_device->disconnect();
		
		channel0Power = -1;
		channel1Power = -1;
		
		set_state(Tango::FAULT);
		set_status("Could not read from the CS410LightController!");
		ERROR_STREAM << "Could not read from the CS410LightController!" << endl;
		
		throw;
	}
	
}

void CS410LightController::flush_read_buffer(Tango::DeviceProxy *dev) throw (Tango::DevFailed)
{
	try
	{
		DEBUG_STREAM << "Flush (start)" << endl;
	
		Tango::DevShort num_chars = 100;
		Tango::DeviceData num_chars_arg;
		num_chars_arg << num_chars;
		string data_read_str;
		do
		{
			Tango::DeviceData data_read = dev->command_inout("DevSerReadRaw", num_chars_arg);
			data_read >> data_read_str;
			
			DEBUG_STREAM << data_read_str << endl;
		} while (!data_read_str.empty());
		
		DEBUG_STREAM << "Flush (end)" << endl;
	}
	catch(...)
	{
		DEBUG_STREAM << "Could not flush the CS410LightController!" << endl;
		
		serial_device->disconnect();
		
		channel0Power = -1;
		channel1Power = -1;
		
		set_state(Tango::FAULT);
		set_status("Could not flush the CS410LightController!");
		ERROR_STREAM << "Could not flush the CS410LightController!" << endl;
		
		throw;
	}
}

void CS410LightController::write_string(Tango::DeviceProxy *dev, const std::string & str) throw (Tango::DevFailed)
{
	try
	{
		DEBUG_STREAM << "Writing (start)" << endl;
		DEBUG_STREAM << "Writing: " << str.c_str() << endl;
		
		Tango::DeviceData line_arg;
		Tango::DevString line_string = CORBA::string_dup(str.c_str());
		line_arg << line_string;
		dev->command_inout("DevSerWriteString", line_arg);
		
		DEBUG_STREAM << "Writing (end)" << endl;
		INFO_STREAM << "Writing: " << str.c_str() << endl;
	}
	catch(...)
	{
		DEBUG_STREAM << "Could not write to the CS410LightController!" << endl;
		
		serial_device->disconnect();
		
		channel0Power = -1;
		channel1Power = -1;
		
		set_state(Tango::FAULT);
		set_status("Could not write to the CS410LightController!");
		
		ERROR_STREAM << "Could not write to the CS410LightController!" << endl;
		
		throw;
	}
}

Tango::DeviceProxy *CS410LightController::get_serial_device() throw (Tango::DevFailed)
{
	if (serial_device->is_connected())
		return serial_device->get();
	
	// not connected, connect, set up comm parameters, and synchronize with the device
	Tango::DeviceProxy *dev = serial_device->get();
	
	// set parity
	Tango::DevShort parity = 0; // NONE parity
	Tango::DeviceData parity_arg;
	parity_arg << parity;
	dev->command_inout("DevSerSetParity", parity_arg);
	
	// set char length
	Tango::DevShort char_len = 0; // char length is 8 bit
	Tango::DeviceData char_len_arg;
	char_len_arg << char_len;
	dev->command_inout("DevSerSetCharLength", char_len_arg);

	// set stop bit
	Tango::DevShort stop_bit = 1; // stop bit is 1 bit
	Tango::DeviceData stop_bit_arg;
	stop_bit_arg << stop_bit;
	dev->command_inout("DevSerSetStopbit", stop_bit_arg);
	
	// set baud rate
	Tango::DevULong baud_rate = 19200; // baud rate 19200
	Tango::DeviceData baud_rate_arg;
	baud_rate_arg << baud_rate;
	dev->command_inout("DevSerSetBaudrate", baud_rate_arg);

	// set new line
	Tango::DevShort new_line = 10; // new line
	Tango::DeviceData new_line_arg;
	new_line_arg << new_line;
	dev->command_inout("DevSerSetNewline", new_line_arg);
	
	// set timeout
	Tango::DevShort timeout = 1000; // timeout
	Tango::DeviceData timeout_arg;
	timeout_arg << timeout;
	dev->command_inout("DevSerSetTimeout", timeout_arg);
	
	// flush the input
	 
	flush_read_buffer(dev);

	// synchronize with the prompt
	string response;
	for (int i = 0; i < 3; i++)
	{
		write_string(dev, COMMAND_OPTION_RESET);
		
		response = read_line(dev);
	}
	
	// check that after 3 times, we did get the "Ok" answer
	
	if (response.size() < RESET_REPLY_LEN || response.at(0) != 'O' || response.at(1) != 'k' || response.at(2) != '\r' || response.at(3) != '\n')
	{
		TangoSys_OMemStream os;
		os << "Error, could not synchronize with the device!";
		DEBUG_STREAM << os.str().c_str() << endl;
				
		set_state(Tango::FAULT);
		set_status(os.str());
		
		Tango::Except::throw_exception("Illegal value", os.str(), __func__); 
	}

	// finally flush input again
	flush_read_buffer(dev);

	// return device
	return dev;
}

const char * CS410LightController::COMMAND_OPTION_RESET = "OR";
const char * CS410LightController::COMMAND_SET_CHANNEL_POWER = "S%1d%04d";
const char * CS410LightController::COMMAND_GET_CHANNEL_POWER = "R%1d";

}	//	namespace
