#include "TangoDeviceConnection.h"

TangoDeviceConnection::TangoDeviceConnection(const std::string & _connect_string): connect_string(_connect_string), device(NULL)
{
	
}

TangoDeviceConnection::~TangoDeviceConnection()
{
	disconnect();
}

Tango::DeviceProxy *TangoDeviceConnection::get() throw (Tango::DevFailed)
{
	if (is_connected())
		return device;
	
	// we're not connected
	
	disconnect(); // so that we do not leak the previous DeviceProxy instance
	
	// try to (re)connect
	// this may throw an exception
	// allow it since it's pointless returning a NULL (the caller would need to check for it everytime) 
	
	device = new Tango::DeviceProxy(connect_string.c_str());
	
	device->ping(); // we want to be sure we're connected
	
	return device;
}

bool TangoDeviceConnection::is_connected() const
{
	if (!device)
	{
		return false;
	}
	else
	{
		try
		{
			device->ping();
			return true;
		}
		catch(...)
		{
			return false;
		}
	}
}

void TangoDeviceConnection::disconnect()
{
	delete device;
	device = NULL;
}
